# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import Case_pb2 as Case__pb2
import Definitions_pb2 as Definitions__pb2
import PdmObject_pb2 as PdmObject__pb2


class ProjectStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetCurrentCase = channel.unary_unary(
                '/rips.Project/GetCurrentCase',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseRequest.FromString,
                )
        self.GetSelectedCases = channel.unary_unary(
                '/rips.Project/GetSelectedCases',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                )
        self.GetAllCaseGroups = channel.unary_unary(
                '/rips.Project/GetAllCaseGroups',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseGroups.FromString,
                )
        self.GetAllCases = channel.unary_unary(
                '/rips.Project/GetAllCases',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                )
        self.GetCasesInGroup = channel.unary_unary(
                '/rips.Project/GetCasesInGroup',
                request_serializer=Case__pb2.CaseGroup.SerializeToString,
                response_deserializer=Case__pb2.CaseInfoArray.FromString,
                )
        self.GetPdmObject = channel.unary_unary(
                '/rips.Project/GetPdmObject',
                request_serializer=Definitions__pb2.Empty.SerializeToString,
                response_deserializer=PdmObject__pb2.PdmObject.FromString,
                )


class ProjectServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetCurrentCase(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSelectedCases(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCaseGroups(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAllCases(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCasesInGroup(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetPdmObject(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_ProjectServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetCurrentCase': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCurrentCase,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseRequest.SerializeToString,
            ),
            'GetSelectedCases': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSelectedCases,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetAllCaseGroups': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCaseGroups,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseGroups.SerializeToString,
            ),
            'GetAllCases': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAllCases,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetCasesInGroup': grpc.unary_unary_rpc_method_handler(
                    servicer.GetCasesInGroup,
                    request_deserializer=Case__pb2.CaseGroup.FromString,
                    response_serializer=Case__pb2.CaseInfoArray.SerializeToString,
            ),
            'GetPdmObject': grpc.unary_unary_rpc_method_handler(
                    servicer.GetPdmObject,
                    request_deserializer=Definitions__pb2.Empty.FromString,
                    response_serializer=PdmObject__pb2.PdmObject.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rips.Project', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Project(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetCurrentCase(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetCurrentCase',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseRequest.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSelectedCases(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetSelectedCases',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllCaseGroups(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetAllCaseGroups',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseGroups.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAllCases(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetAllCases',
            Definitions__pb2.Empty.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCasesInGroup(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetCasesInGroup',
            Case__pb2.CaseGroup.SerializeToString,
            Case__pb2.CaseInfoArray.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetPdmObject(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Project/GetPdmObject',
            Definitions__pb2.Empty.SerializeToString,
            PdmObject__pb2.PdmObject.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



######################################################################
# This script sets values for all grid cells in the first case in the project
# The script is intended to be used for TEST10K_FLT_LGR_NNC.EGRID
# This grid case contains one LGR
######################################################################
import rips

resinsight = rips.Instance.find()

case = resinsight.project.case(case_id=0)
grid = case.grid()
grid_cell_count = grid.cell_count()
print("total cell count : " + str(grid_cell_count))

values = []
for i in range(0, grid_cell_count):
    values.append(i % 2 * 0.75)

# Assign value to IJK grid cell at (31, 53, 21)
grid = case.grid()
property_data_index = grid.property_data_index_from_ijk(31, 53, 21)
values[property_data_index] = 1.5

print("Applying values to main grid")
case.set_grid_property(values, "STATIC_NATIVE", "MY_DATA", 0)

values_from_ri = case.grid_property("STATIC_NATIVE", "MY_DATA", 0)
assert values[property_data_index] == values_from_ri[property_data_index]

# Get LGR grid as grid index 1
grid = case.grid(1)
grid_cell_count = grid.cell_count()
print("lgr cell count : " + str(grid_cell_count))

values = []
for i in range(0, grid_cell_count):
    values.append(i % 3 * 0.75)

print("Applying values to LGR grid")
case.set_grid_property(values, "STATIC_NATIVE", "MY_DATA", 0, 1)
values_from_ri = case.grid_property("STATIC_NATIVE", "MY_DATA", 0, 1)



"""
Grid Case Group statistics module
"""

from .pdmobject import add_method
from .view import View
from .case import Case

import Commands_pb2
from .resinsight_classes import GridCaseGroup
from .resinsight_classes import EclipseView


@add_method(GridCaseGroup)
def create_statistics_case(self):
    """Create a Statistics case in the Grid Case Group

    Returns:
        :class:`rips.generated.generated_classes.EclipseCase`
    """
    command_reply = self._execute_command(
        createStatisticsCase=Commands_pb2.CreateStatisticsCaseRequest(
            caseGroupId=self.group_id
        )
    )
    return Case(self.channel, command_reply.createStatisticsCaseResult.caseId)


@add_method(GridCaseGroup)
def statistics_cases(self):
    """Get a list of all statistics cases in the Grid Case Group

    Returns:
        List of :class:`rips.generated.generated_classes.EclipseCase`

    """
    stat_case_collection = self.children("StatisticsCaseCollection")[0]
    return stat_case_collection.children("Reservoirs")


@add_method(GridCaseGroup)
def views(self):
    """Get a list of views belonging to a grid case group

    Returns:
        List of :class:`rips.generated.generated_classes.EclipseView`

    """
    resinsight_classes = self.descendants(EclipseView)
    view_list = []
    for pdm_object in resinsight_classes:
        view_list.append(pdm_object)
    return view_list


@add_method(GridCaseGroup)
def view(self, view_id):
    """Get a particular view belonging to a case group by providing view id

    Arguments:
        id(int): view id

    Returns:
        List of :class:`rips.generated.generated_classes.EclipseView`

    """
    views = self.views()
    for view_object in views:
        if view_object.id == view_id:
            return view_object
    return None


@add_method(GridCaseGroup)
def compute_statistics(self, case_ids=None):
    """Compute statistics for the given case ids

    Arguments:
        case_ids(list of integers): List of case ids. If this is None all cases in group are included

    """
    if case_ids is None:
        case_ids = []
    return self._execute_command(
        computeCaseGroupStatistics=Commands_pb2.ComputeCaseGroupStatRequest(
            caseIds=case_ids, caseGroupId=self.group_id
        )
    )



###########################################################################################
# This example will asynchronously calculate the average value for SOIL for all time steps
###########################################################################################

import rips
import itertools
import time

resinsight = rips.Instance.find()

start = time.time()

# Get the case with case id 0
case = resinsight.project.case(case_id=0)

# Get a list of all time steps
timeSteps = case.time_steps()

averages = []
for i in range(0, len(timeSteps)):
    # Get the results from time step i asynchronously
    # It actually returns a generator object almost immediately
    result_chunks = case.active_cell_property_async("DYNAMIC_NATIVE", "SOIL", i)
    mysum = 0.0
    count = 0
    # Loop through and append the average. each time we loop resultChunks
    # We will trigger a read of the input data, meaning the script will start
    # Calculating averages before the whole resultValue for this time step has been received
    for chunk in result_chunks:
        mysum += sum(chunk.values)
        count += len(chunk.values)

    averages.append(mysum / count)

end = time.time()
print("Time elapsed: ", end - start)
print(averages)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import Definitions_pb2 as Definitions__pb2
import Grid_pb2 as Grid__pb2


class GridStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetCellCenters = channel.unary_stream(
                '/rips.Grid/GetCellCenters',
                request_serializer=Grid__pb2.GridRequest.SerializeToString,
                response_deserializer=Definitions__pb2.CellCenters.FromString,
                )
        self.GetCellCorners = channel.unary_stream(
                '/rips.Grid/GetCellCorners',
                request_serializer=Grid__pb2.GridRequest.SerializeToString,
                response_deserializer=Definitions__pb2.CellCornersArray.FromString,
                )
        self.GetDimensions = channel.unary_unary(
                '/rips.Grid/GetDimensions',
                request_serializer=Grid__pb2.GridRequest.SerializeToString,
                response_deserializer=Grid__pb2.GridDimensions.FromString,
                )


class GridServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetCellCenters(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetCellCorners(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetDimensions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_GridServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetCellCenters': grpc.unary_stream_rpc_method_handler(
                    servicer.GetCellCenters,
                    request_deserializer=Grid__pb2.GridRequest.FromString,
                    response_serializer=Definitions__pb2.CellCenters.SerializeToString,
            ),
            'GetCellCorners': grpc.unary_stream_rpc_method_handler(
                    servicer.GetCellCorners,
                    request_deserializer=Grid__pb2.GridRequest.FromString,
                    response_serializer=Definitions__pb2.CellCornersArray.SerializeToString,
            ),
            'GetDimensions': grpc.unary_unary_rpc_method_handler(
                    servicer.GetDimensions,
                    request_deserializer=Grid__pb2.GridRequest.FromString,
                    response_serializer=Grid__pb2.GridDimensions.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'rips.Grid', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Grid(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetCellCenters(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rips.Grid/GetCellCenters',
            Grid__pb2.GridRequest.SerializeToString,
            Definitions__pb2.CellCenters.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetCellCorners(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/rips.Grid/GetCellCorners',
            Grid__pb2.GridRequest.SerializeToString,
            Definitions__pb2.CellCornersArray.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetDimensions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/rips.Grid/GetDimensions',
            Grid__pb2.GridRequest.SerializeToString,
            Grid__pb2.GridDimensions.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()
project = resinsight.project

# Look for input files in the home directory of the user
home_dir = expanduser("~")
fracture_file_path = (Path(home_dir) / "fracture.csv").as_posix()
print("Thermal fracture file path:", fracture_file_path)

# Find a case
cases = resinsight.project.cases()
case = cases[0]

# Create thermal template
fmt_collection = project.descendants(rips.FractureTemplateCollection)[0]
fracture_template = fmt_collection.append_thermal_fracture_template(
    file_path=fracture_file_path
)

well_name = "F-1 H"

# Find a well
well_path = project.well_path_by_name(well_name)
print("Well path:", well_path.name)

# Create fracture and place it using data from the fracture template
fracture = well_path.add_thermal_fracture(
    fracture_template=fracture_template,
    place_using_template_data=True,
)


time_steps = fracture_template.time_steps().values
for time_step_index, time_stamp in enumerate(time_steps):
    print("Time step #{}: {}".format(time_step_index, time_stamp))
    fracture_template.active_time_step_index = time_step_index
    fracture_template.conductivity_result_name = "Conductivity"
    fracture_template.update()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Grid.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Definitions_pb2 as Definitions__pb2
import Case_pb2 as Case__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nGrid.proto\x12\x04rips\x1a\x11\x44\x65\x66initions.proto\x1a\nCase.proto\"J\n\x0bGridRequest\x12\'\n\x0c\x63\x61se_request\x18\x01 \x01(\x0b\x32\x11.rips.CaseRequest\x12\x12\n\ngrid_index\x18\x02 \x01(\x05\"1\n\x0eGridDimensions\x12\x1f\n\ndimensions\x18\x01 \x01(\x0b\x32\x0b.rips.Vec3i2\xbf\x01\n\x04Grid\x12:\n\x0eGetCellCenters\x12\x11.rips.GridRequest\x1a\x11.rips.CellCenters\"\x00\x30\x01\x12?\n\x0eGetCellCorners\x12\x11.rips.GridRequest\x1a\x16.rips.CellCornersArray\"\x00\x30\x01\x12:\n\rGetDimensions\x12\x11.rips.GridRequest\x1a\x14.rips.GridDimensions\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Grid_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_GRIDREQUEST']._serialized_start=51
  _globals['_GRIDREQUEST']._serialized_end=125
  _globals['_GRIDDIMENSIONS']._serialized_start=127
  _globals['_GRIDDIMENSIONS']._serialized_end=176
  _globals['_GRID']._serialized_start=179
  _globals['_GRID']._serialized_end=370
# @@protoc_insertion_point(module_scope)



import os
import rips

resinsight = rips.Instance.find()

case_paths = []
case_paths.append(
    "C:/Users/lindk/source/repos/ResInsight/TestModels/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
)
case_paths.append(
    "C:/Users/lindk/source/repos/ResInsight/TestModels/Case_with_10_timesteps/Real10/BRUGGE_0010.EGRID"
)
for case_path in case_paths:
    assert os.path.exists(
        case_path
    ), "You need to set valid case paths for this script to work"

case_group = resinsight.project.create_grid_case_group(case_paths=case_paths)

case_group.compute_statistics()

view = case_group.views()[0]
view.apply_cell_result("DYNAMIC_NATIVE", "PRESSURE_DEV")



import sys
import os
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_10k_well_log_extraction(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    view = case.create_view()
    view.set_time_step(1)

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    properties = [
        ("STATIC_NATIVE", "INDEX_K", 0),
        ("STATIC_NATIVE", "PORO", 0),
        ("STATIC_NATIVE", "PERMX", 0),
        ("DYNAMIC_NATIVE", "PRESSURE", 0),
    ]

    well_log_plot_collection = rips_instance.project.descendants(
        rips.WellLogPlotCollection
    )[0]

    well_log_plot = well_log_plot_collection.new_well_log_plot(case, well_path)

    # Create a track for each property
    for prop_type, prop_name, time_step in properties:
        track = well_log_plot.new_well_log_track("Track: " + prop_name, case, well_path)
        c = track.add_extraction_curve(case, well_path, prop_type, prop_name, time_step)

    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        well_log_plot.export_data_as_las(export_folder=tmpdirname)



########################################################################################
# This example generates a derived property in an asynchronous manner
# Meaning it does not wait for all the data for each stage to be read before proceeding
########################################################################################
import rips
import time


# Internal function for creating a result from a small chunk of poro and permx results
# The return value of the function is a generator for the results rather than the result itself.
def create_result(poro_chunks, permx_chunks):
    # Loop through all the chunks of poro and permx in order
    for poroChunk, permxChunk in zip(poro_chunks, permx_chunks):
        resultChunk = []
        # Loop through all the values inside the chunks, in order
        for poro, permx in zip(poroChunk.values, permxChunk.values):
            resultChunk.append(poro * permx)
        # Return a generator object that behaves like a Python iterator
        yield resultChunk


resinsight = rips.Instance.find()
start = time.time()
case = resinsight.project.cases()[0]

# Get a generator for the poro results. The generator will provide a chunk each time it is iterated
poro_chunks = case.active_cell_property_async("STATIC_NATIVE", "PORO", 0)
# Get a generator for the permx results. The generator will provide a chunk each time it is iterated
permx_chunks = case.active_cell_property_async("STATIC_NATIVE", "PERMX", 0)

# Send back the result with the result provided by a generator object.
# Iterating the result generator will cause the script to read from the poro and permx generators
# And return the result of each iteration
case.set_active_cell_property_async(
    create_result(poro_chunks, permx_chunks), "GENERATED", "POROPERMXAS", 0
)

end = time.time()
print("Time elapsed: ", end - start)
print("Transferred all results back")
view = case.views()[0].apply_cell_result("GENERATED", "POROPERMXAS")



########################################################################################
# This example generates a derived property in an synchronous manner
# Meaning it completes reading each result before calculating the derived result
# See InputPropTestAsync for how to do this asynchronously instead.
########################################################################################
import rips
import time
import grpc

resinsight = rips.Instance.find()
start = time.time()
case = resinsight.project.cases()[0]

# Read poro result into list
poro_results = case.active_cell_property("STATIC_NATIVE", "PORO", 0)
# Read permx result into list
permx_results = case.active_cell_property("STATIC_NATIVE", "PERMX", 0)

# Generate output result
results = []
for poro, permx in zip(poro_results, permx_results):
    results.append(poro * permx)

try:
    # Send back output result
    case.set_active_cell_property(results, "GENERATED", "POROPERMXSY", 0)
except grpc.RpcError as e:
    print("Exception Received: ", e)


end = time.time()
print("Time elapsed: ", end - start)
print("Transferred all results back")

view = case.views()[0].apply_cell_result("GENERATED", "POROPERMXSY")



import os
import grpc

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resInsight = rips.Instance.find()

# Get all GeoMech cases
cases = resInsight.project.descendants(rips.GeoMechCase)

# Get all well paths
well_paths = resInsight.project.well_paths()

# Ensure there's at least one well path
if len(well_paths) < 1:
    print("No well paths in project")
    exit(1)

# Create a set of WbsParameters
params = rips.WbsParameters()
params.user_poisson_ratio = 0.23456
params.user_ucs = 123

# Loop through all cases
for case in cases:
    assert isinstance(case, rips.GeoMechCase)
    min_res_depth, max_res_depth = case.reservoir_depth_range()

    # Find a good output path
    case_path = case.file_path
    folder_name = os.path.dirname(case_path)

    # Import formation names
    case.import_formation_names(
        formation_files=[
            "D:/Projects/ResInsight-regression-test/ModelData/norne/Norne_ATW2013.lyr"
        ]
    )

    # create a folder to hold the snapshots
    dirname = os.path.join(folder_name, "snapshots")
    print("Exporting to: " + dirname)

    for well_path in well_paths[0:4]:  # Loop through the first five well paths
        # Create plot with parameters
        wbsplot = case.create_well_bore_stability_plot(
            well_path=well_path.name, time_step=0, parameters=params
        )



import sys
import os
import pytest
import grpc
import tempfile

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_loadProject(rips_instance, initialize_test):
    project = rips_instance.project.open(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/10KWithWellLog.rsp"
    )
    case = project.cases()[0]
    assert case is not None
    assert case.name == "TEST10K_FLT_LGR_NNC"
    assert case.id == 0
    cases = rips_instance.project.cases()
    assert len(cases) is 1


def test_well_log_plots(rips_instance, initialize_test):
    project = rips_instance.project.open(
        dataroot.PATH + "/TEST10K_FLT_LGR_NNC/10KWithWellLog.rsp"
    )
    plots = project.plots()
    well_log_plots = []
    for plot in plots:
        if isinstance(plot, rips.WellLogPlot):
            assert plot.depth_type == "MEASURED_DEPTH"
            well_log_plots.append(plot)
    assert len(well_log_plots) == 2

    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        for well_log_plot in well_log_plots:
            well_log_plot.depth_type = "TRUE_VERTICAL_DEPTH_RKB"
            well_log_plot.update()
            if rips_instance.is_gui():
                well_log_plot.export_snapshot(tmpdirname)
            well_log_plot.export_data_as_las(tmpdirname)
        files = os.listdir(tmpdirname)
        print(files)
        if rips_instance.is_gui():
            assert len(files) == 4
        else:
            assert len(files) == 2

    plots2 = project.plots()
    for plot2 in plots2:
        if isinstance(plot2, rips.WellLogPlot):
            assert plot2.depth_type == "TRUE_VERTICAL_DEPTH_RKB"


@pytest.mark.skipif(
    sys.platform.startswith("linux"),
    reason="Brugge is currently exceptionally slow on Linux",
)
def test_loadGridCaseGroup(rips_instance, initialize_test):
    case_paths = []
    case_paths.append(dataroot.PATH + "/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID")
    case_paths.append(
        dataroot.PATH + "/Case_with_10_timesteps/Real10/BRUGGE_0010.EGRID"
    )
    grid_case_group = rips_instance.project.create_grid_case_group(
        case_paths=case_paths
    )
    assert grid_case_group is not None and grid_case_group.group_id == 0


def test_exportSnapshots(rips_instance, initialize_test):
    if not rips_instance.is_gui():
        pytest.skip("Cannot run test without a GUI")

    case_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
    rips_instance.project.load_case(case_path)
    with tempfile.TemporaryDirectory(prefix="rips") as tmpdirname:
        print("Temporary folder: ", tmpdirname)
        rips_instance.set_export_folder(export_type="SNAPSHOTS", path=tmpdirname)
        rips_instance.project.export_snapshots()
        print(os.listdir(tmpdirname))
        #        assert(len(os.listdir(tmpdirname)) > 0)
        for fileName in os.listdir(tmpdirname):
            assert os.path.splitext(fileName)[1] == ".png"



# Load ResInsight Processing Server Client Library
import rips

# Launch ResInsight with last project file and a Window size of 600x1000 pixels
resinsight = rips.Instance.launch(
    command_line_parameters=["--last", "--size", 600, 1000]
)
# Get a list of all cases
cases = resinsight.project.cases()

print("Got " + str(len(cases)) + " cases: ")
for case in cases:
    print("Case name: " + case.name)
    print("Case grid path: " + case.file_path)



############################################################################
# This example returns the currently selected cases in ResInsight
# Because running this script in the GUI takes away the selection
# This script does not run successfully from within the ResInsight GUI
# And will need to be run from the command line separately from ResInsight
############################################################################

import rips

resinsight = rips.Instance.find()
if resinsight is not None:
    cases = resinsight.project.selected_cases()

    print("Got " + str(len(cases)) + " cases: ")
    for case in cases:
        print(case.name)
        for property in case.available_properties("DYNAMIC_NATIVE"):
            print(property)



import sys
import os

import rips

use_platform_offscreen = True
if use_platform_offscreen:
    # To use offscreen, the path to fonts must be specified in the environment variable QT_QPA_FONTDIR="C:/windows/fonts"
    resinsight = rips.Instance.launch(
        command_line_parameters=["-platform", "offscreen", "--size", 1200, 1000]
    )

    qpa_fontdir = os.environ["QT_QPA_FONTDIR"]
    print("Environment var QT_QPA_FONTDIR : " + qpa_fontdir)
else:
    resinsight = rips.Instance.find()

summary_filename = "NORNE.SMSPEC"

project = resinsight.project
summary_case = project.import_summary_case(summary_filename)

summary_plot_collection = project.descendants(rips.SummaryPlotCollection)[0]

summary_plot_collection.new_summary_plot(summary_cases=[summary_case], address="FOPR")
summary_plot_collection.new_summary_plot(
    summary_cases=[summary_case], address="WOPR:A*;WOPR:B*"
)

plots = resinsight.project.plots()
for plot in plots:
    plot.export_snapshot()
    # plot.export_snapshot(output_format="PDF")

resinsight.exit()



# Load ResInsight Processing Server Client Library
import rips
import tempfile
from os.path import expanduser
from pathlib import Path

# Connect to ResInsight instance
resinsight = rips.Instance.find()
project = resinsight.project

# Look for input files in the home directory of the user
home_dir = expanduser("~")
stim_plan_file_path = (Path(home_dir) / "contour.xml").as_posix()
print("StimPlan contour file path:", stim_plan_file_path)

# Find a case
cases = resinsight.project.cases()
case = cases[0]

# Create stim plan template
fmt_collection = project.descendants(rips.FractureTemplateCollection)[0]
fracture_template = fmt_collection.append_fracture_template(
    file_path=stim_plan_file_path
)

well_name = "B-2 H"

# Find a well
well_path = project.well_path_by_name(well_name)
print("well path:", well_path.name)

# Place fracture at given depths
measured_depths = [3200.0, 3400.0, 3600.0]
for measured_depth in measured_depths:
    print("Placing fracture at {} depth (MD)".format(measured_depth))
    # Create stim plan  at a give measured depth
    fracture = well_path.add_fracture(
        measured_depth=measured_depth,
        stim_plan_fracture_template=fracture_template,
        align_dip=True,
        eclipse_case=case,
    )

# Update the orientation of the fracture
# Call update() to propagate changes from the Python object back to ResInsight
fracture_template.orientation = "Azimuth"
fracture_template.azimuth_angle = 60.0
fracture_template.user_defined_perforation_length = True
fracture_template.conductivity_type = "InfiniteConductivity"
fracture_template.perforation_length = 12.3
fracture_template.update()

# Scale the template
fracture_template.set_scale_factors(
    half_length=2.0, height=2.0, d_factor=1.1, conductivity=1.2
)

# Output scale factors for all fracture templates
fmt_collection = project.descendants(rips.FractureTemplate)
for fracture_template in fmt_collection:
    print(
        "Fracture: '{}' Scale factors: Height={} Half Length={} D Factor={} Conductivity={}".format(
            fracture_template.user_description,
            fracture_template.height_scale_factor,
            fracture_template.width_scale_factor,
            fracture_template.d_factor_scale_factor,
            fracture_template.conductivity_factor,
        )
    )



############################################################################
# This script will export completions for a well path for all cases in the project
#
############################################################################

import os
import rips

# Load instance
resinsight = rips.Instance.find()
cases = resinsight.project.cases()

for case in cases:
    print("Case name: ", case.name)
    print("Case id: ", case.id)

    case.export_well_path_completions(
        time_step=0,
        well_path_names=["Well-1"],
        file_split="UNIFIED_FILE",
        include_perforations=True,
        custom_file_name="d:/scratch/well_path_export/myfile.myext",
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Properties.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import Definitions_pb2 as Definitions__pb2
import Case_pb2 as Case__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10Properties.proto\x12\x04rips\x1a\x11\x44\x65\x66initions.proto\x1a\nCase.proto\"\xa1\x01\n\x1a\x41vailablePropertiesRequest\x12\'\n\x0c\x63\x61se_request\x18\x01 \x01(\x0b\x32\x11.rips.CaseRequest\x12)\n\rproperty_type\x18\x02 \x01(\x0e\x32\x12.rips.PropertyType\x12/\n\x0eporosity_model\x18\x03 \x01(\x0e\x32\x17.rips.PorosityModelType\"-\n\x13\x41vailableProperties\x12\x16\n\x0eproperty_names\x18\x01 \x03(\t\"\xd4\x01\n\x0fPropertyRequest\x12\'\n\x0c\x63\x61se_request\x18\x01 \x01(\x0b\x32\x11.rips.CaseRequest\x12)\n\rproperty_type\x18\x02 \x01(\x0e\x32\x12.rips.PropertyType\x12\x15\n\rproperty_name\x18\x03 \x01(\t\x12\x11\n\ttime_step\x18\x04 \x01(\x05\x12\x12\n\ngrid_index\x18\x05 \x01(\x05\x12/\n\x0eporosity_model\x18\x06 \x01(\x0e\x32\x17.rips.PorosityModelType\"\x19\n\x08TimeStep\x12\r\n\x05index\x18\x01 \x01(\x05\"q\n\x12PropertyInputChunk\x12\'\n\x06params\x18\x01 \x01(\x0b\x32\x15.rips.PropertyRequestH\x00\x12%\n\x06values\x18\x02 \x01(\x0b\x32\x13.rips.PropertyChunkH\x00\x42\x0b\n\tChunkType\"\x1f\n\rPropertyChunk\x12\x0e\n\x06values\x18\x01 \x03(\x01*\xc7\x01\n\x0cPropertyType\x12\x12\n\x0e\x44YNAMIC_NATIVE\x10\x00\x12\x11\n\rSTATIC_NATIVE\x10\x01\x12\r\n\tSOURSIMRL\x10\x02\x12\r\n\tGENERATED\x10\x03\x12\x12\n\x0eINPUT_PROPERTY\x10\x04\x12\x13\n\x0f\x46ORMATION_NAMES\x10\x05\x12\x14\n\x10\x46LOW_DIAGNOSTICS\x10\x06\x12\x16\n\x12INJECTION_FLOODING\x10\x07\x12\x0b\n\x07REMOVED\x10\x08\x12\x0e\n\tUNDEFINED\x10\xe7\x07\x32\xe6\x03\n\nProperties\x12W\n\x16GetAvailableProperties\x12 .rips.AvailablePropertiesRequest\x1a\x19.rips.AvailableProperties\"\x00\x12G\n\x15GetActiveCellProperty\x12\x15.rips.PropertyRequest\x1a\x13.rips.PropertyChunk\"\x00\x30\x01\x12I\n\x17GetSelectedCellProperty\x12\x15.rips.PropertyRequest\x1a\x13.rips.PropertyChunk\"\x00\x30\x01\x12\x41\n\x0fGetGridProperty\x12\x15.rips.PropertyRequest\x1a\x13.rips.PropertyChunk\"\x00\x30\x01\x12V\n\x15SetActiveCellProperty\x12\x18.rips.PropertyInputChunk\x1a\x1f.rips.ClientToServerStreamReply\"\x00(\x01\x12P\n\x0fSetGridProperty\x12\x18.rips.PropertyInputChunk\x1a\x1f.rips.ClientToServerStreamReply\"\x00(\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Properties_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_PROPERTYTYPE']._serialized_start=659
  _globals['_PROPERTYTYPE']._serialized_end=858
  _globals['_AVAILABLEPROPERTIESREQUEST']._serialized_start=58
  _globals['_AVAILABLEPROPERTIESREQUEST']._serialized_end=219
  _globals['_AVAILABLEPROPERTIES']._serialized_start=221
  _globals['_AVAILABLEPROPERTIES']._serialized_end=266
  _globals['_PROPERTYREQUEST']._serialized_start=269
  _globals['_PROPERTYREQUEST']._serialized_end=481
  _globals['_TIMESTEP']._serialized_start=483
  _globals['_TIMESTEP']._serialized_end=508
  _globals['_PROPERTYINPUTCHUNK']._serialized_start=510
  _globals['_PROPERTYINPUTCHUNK']._serialized_end=623
  _globals['_PROPERTYCHUNK']._serialized_start=625
  _globals['_PROPERTYCHUNK']._serialized_end=656
  _globals['_PROPERTIES']._serialized_start=861
  _globals['_PROPERTIES']._serialized_end=1347
# @@protoc_insertion_point(module_scope)



"""
ResInsight SimulationWell
"""
import grpc

import SimulationWell_pb2
import SimulationWell_pb2_grpc

import Properties_pb2
import Properties_pb2_grpc

from .resinsight_classes import SimulationWell

from .pdmobject import PdmObjectBase, add_method

import rips.case


@add_method(SimulationWell)
def __custom_init__(self, pb2_object, channel):
    self._simulation_well_stub = SimulationWell_pb2_grpc.SimulationWellStub(channel)


@add_method(SimulationWell)
def status(self, timestep):
    """Get simulation well status

     **SimulationWellStatus class description**::

        Parameter   | Description                                                   | Type
        ----------- | ------------------------------------------------------------- | -----
        well_type   | Well type as string                                           | string
        is_open     | True if simulation well is open at the specified time step    | bool

    Arguments:
        timestep(int): Time step index

    """
    sim_well_request = SimulationWell_pb2.SimulationWellRequest(
        case_id=self.case().id, well_name=self.name, timestep=timestep
    )
    return self._simulation_well_stub.GetSimulationWellStatus(sim_well_request)


@add_method(SimulationWell)
def cells(self, timestep):
    """Get reservoir cells the simulation well is defined for

     **SimulationWellCellInfo class description**::

        Parameter   | Description                                               | Type
        ----------- | --------------------------------------------------------- | -----
        ijk         | Cell IJK location                                         | Vec3i
        grid_index  | Grid index                                                | int
        is_open     | True if connection to is open at the specified time step  | bool
        branch_id   |                                                           | int
        segment_id  |                                                           | int

    Arguments:
        timestep(int): Time step index

    Returns:
        List of SimulationWellCellInfo

    """
    sim_well_request = SimulationWell_pb2.SimulationWellRequest(
        case_id=self.case().id, well_name=self.name, timestep=timestep
    )
    return self._simulation_well_stub.GetSimulationWellCells(sim_well_request).data


@add_method(SimulationWell)
def case(self):
    return self.ancestor(rips.case.Case)



import sys
import os

sys.path.insert(1, os.path.join(sys.path[0], "../../"))
import rips

import dataroot


def test_10k(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
        case_root_path + "/wellpath_b.dev",
    ]
    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    wells = rips_instance.project.well_paths()
    assert len(wells) == 2
    assert wells[0].name == "Well Path A"
    assert wells[1].name == "Well Path B"


def test_10k_intersection(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)
    assert len(case.grids()) == 2
    well_path_files = [
        case_root_path + "/wellpath_a.dev",
    ]

    view = case.create_view()
    view.set_time_step(1)

    well_path_names = rips_instance.project.import_well_paths(well_path_files)
    wells = rips_instance.project.well_paths()
    well_path = wells[0]

    # Add a curve intersection based on the well path
    intersection_coll = rips_instance.project.descendants(rips.IntersectionCollection)[
        0
    ]
    well_path_intersection = intersection_coll.add_new_object(rips.CurveIntersection)
    well_path_intersection.type = "CS_WELL_PATH"
    well_path_intersection.well_path = well_path
    well_path_intersection.update()

    # Three coords per triangle
    geometry = well_path_intersection.geometry()
    coord_count = len(geometry.x_coords)
    assert coord_count == 13254

    # One value per triangle
    geometry_result_values = well_path_intersection.geometry_result()
    result_count = len(geometry_result_values.values)
    assert result_count == 4418

    # Three coords per triangle, one result value per triangle
    assert (result_count * 3) == coord_count


def test_empty_well_intersection(rips_instance, initialize_test):
    case_root_path = dataroot.PATH + "/TEST10K_FLT_LGR_NNC"
    case_path = case_root_path + "/TEST10K_FLT_LGR_NNC.EGRID"
    case = rips_instance.project.load_case(path=case_path)

    view = case.create_view()
    view.set_time_step(1)

    # Add a curve intersection based on the well path
    intersection_coll = rips_instance.project.descendants(rips.IntersectionCollection)[
        0
    ]

    well_path_intersection = intersection_coll.add_new_object(rips.CurveIntersection)
    well_path_intersection.type = "CS_WELL_PATH"
    well_path_intersection.well_path = None
    well_path_intersection.update()

    # Test with empty geometry. This will also test that an empty list in CAF is converted to an empty list in Python
    # See __makelist in pdmobject.py
    geometry = well_path_intersection.geometry()
    coord_count = len(geometry.x_coords)
    assert coord_count == 0

    # One value per triangle
    geometry_result_values = well_path_intersection.geometry_result()
    result_count = len(geometry_result_values.values)
    assert result_count == 0



# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()

# Create a modeled well path and add well path targets
# The coordinates are based on the Norne case

well_path_coll = resinsight.project.descendants(rips.WellPathCollection)[0]
well_path = well_path_coll.add_new_object(rips.ModeledWellPath)
well_path.name = "Test Well-1"
well_path.update()

geometry = well_path.well_path_geometry()

reference_point = geometry.reference_point
reference_point[0] = 457196
reference_point[1] = 7322270
reference_point[2] = 2742
geometry.update()  # Commit updates back to ResInsight

# Create the first well target at the reference point
coord = [0, 0, 0]
geometry.append_well_target(coord)

# Append new well targets relative the the reference point
coord = [454.28, 250, -10]
target = geometry.append_well_target(coord)

coord = [1054.28, 250, -50]
target = geometry.append_well_target(coord)

well_path.append_perforation_interval(3300, 3350, 0.2, 0.76)



############################################################################
# This example prints center and corners for the currently selected cells
# in ResInsight
############################################################################

import rips

resinsight = rips.Instance.find()
if resinsight is not None:
    cases = resinsight.project.cases()

    print("Got " + str(len(cases)) + " cases: ")
    for case in cases:
        print(case.name)
        cells = case.selected_cells()
        print("Found " + str(len(cells)) + " selected cells")

        time_step_info = case.time_steps()

        for idx, cell in enumerate(cells):
            print(
                "Selected cell: [{}, {}, {}] grid: {}".format(
                    cell.ijk.i + 1, cell.ijk.j + 1, cell.ijk.k + 1, cell.grid_index
                )
            )

            # Get the grid and dimensions
            grid = case.grids()[cell.grid_index]
            dimensions = grid.dimensions()

            # Map ijk to cell index
            cell_index = (
                dimensions.i * dimensions.j * cell.ijk.k
                + dimensions.i * cell.ijk.j
                + cell.ijk.i
            )

            # Print the cell center
            cell_centers = grid.cell_centers()
            cell_center = cell_centers[cell_index]
            print(
                "Cell center: [{}, {}, {}]".format(
                    cell_center.x, cell_center.y, cell_center.z
                )
            )

            # Print the cell corners
            cell_corners = grid.cell_corners()[cell_index]
            print("Cell corners:")
            print("c0:\n" + str(cell_corners.c0))
            print("c1:\n" + str(cell_corners.c1))
            print("c2:\n" + str(cell_corners.c2))
            print("c3:\n" + str(cell_corners.c3))
            print("c4:\n" + str(cell_corners.c4))
            print("c5:\n" + str(cell_corners.c5))
            print("c6:\n" + str(cell_corners.c6))
            print("c7:\n" + str(cell_corners.c7))

            for tidx, timestep in enumerate(time_step_info):
                # Read the full SOIL result for time step
                soil_results = case.selected_cell_property(
                    "DYNAMIC_NATIVE", "SOIL", tidx
                )
                print(
                    "SOIL: {} ({}.{}.{})".format(
                        soil_results[idx], timestep.year, timestep.month, timestep.day
                    )
                )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Case.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import PdmObject_pb2 as PdmObject__pb2
import Definitions_pb2 as Definitions__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\nCase.proto\x12\x04rips\x1a\x0fPdmObject.proto\x1a\x11\x44\x65\x66initions.proto\"\x19\n\x0b\x43\x61seRequest\x12\n\n\x02id\x18\x01 \x01(\x05\"D\n\x08\x43\x61seInfo\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x10\n\x08group_id\x18\x02 \x01(\x05\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x0c\n\x04type\x18\x04 \x01(\t\"-\n\rCaseInfoArray\x12\x1c\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x0e.rips.CaseInfo\"g\n\x0b\x42oundingBox\x12\r\n\x05min_x\x18\x01 \x01(\x01\x12\r\n\x05max_x\x18\x02 \x01(\x01\x12\r\n\x05min_y\x18\x03 \x01(\x01\x12\r\n\x05max_y\x18\x04 \x01(\x01\x12\r\n\x05min_z\x18\x05 \x01(\x01\x12\r\n\x05max_z\x18\x06 \x01(\x01\"%\n\tCaseGroup\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\"2\n\nCaseGroups\x12$\n\x0b\x63\x61se_groups\x18\x01 \x03(\x0b\x32\x0f.rips.CaseGroup\"\x1a\n\tGridCount\x12\r\n\x05\x63ount\x18\x01 \x01(\x05\"D\n\tCellCount\x12\x19\n\x11\x61\x63tive_cell_count\x18\x01 \x01(\x05\x12\x1c\n\x14reservoir_cell_count\x18\x02 \x01(\x05\"k\n\x0f\x43\x65llInfoRequest\x12\'\n\x0c\x63\x61se_request\x18\x01 \x01(\x0b\x32\x11.rips.CaseRequest\x12/\n\x0eporosity_model\x18\x02 \x01(\x0e\x32\x17.rips.PorosityModelType\"-\n\rCellInfoArray\x12\x1c\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x0e.rips.CellInfo\"\x98\x01\n\x08\x43\x65llInfo\x12\x12\n\ngrid_index\x18\x01 \x01(\x05\x12\x19\n\x11parent_grid_index\x18\x02 \x01(\x05\x12\x1c\n\x14\x63oarsening_box_index\x18\x03 \x01(\x05\x12\x1e\n\tlocal_ijk\x18\x04 \x01(\x0b\x32\x0b.rips.Vec3i\x12\x1f\n\nparent_ijk\x18\x05 \x01(\x0b\x32\x0b.rips.Vec3i\"9\n\x13\x43oarseningInfoArray\x12\"\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x14.rips.CoarseningInfo\"D\n\x0e\x43oarseningInfo\x12\x18\n\x03min\x18\x01 \x01(\x0b\x32\x0b.rips.Vec3i\x12\x18\n\x03max\x18\x02 \x01(\x0b\x32\x0b.rips.Vec3i\"2\n\rTimeStepDates\x12!\n\x05\x64\x61tes\x18\x01 \x03(\x0b\x32\x12.rips.TimeStepDate\"f\n\x0cTimeStepDate\x12\x0c\n\x04year\x18\x01 \x01(\x05\x12\r\n\x05month\x18\x02 \x01(\x05\x12\x0b\n\x03\x64\x61y\x18\x03 \x01(\x05\x12\x0c\n\x04hour\x18\x04 \x01(\x05\x12\x0e\n\x06minute\x18\x05 \x01(\x05\x12\x0e\n\x06second\x18\x06 \x01(\x05\"&\n\x0e\x44\x61ysSinceStart\x12\x14\n\x0c\x64\x61y_decimals\x18\x01 \x03(\x01\"<\n\x0cSelectedCell\x12\x12\n\ngrid_index\x18\x01 \x01(\x05\x12\x18\n\x03ijk\x18\x02 \x01(\x0b\x32\x0b.rips.Vec3i\"2\n\rSelectedCells\x12!\n\x05\x63\x65lls\x18\x01 \x03(\x0b\x32\x12.rips.SelectedCell*9\n\x11PorosityModelType\x12\x10\n\x0cMATRIX_MODEL\x10\x00\x12\x12\n\x0e\x46RACTURE_MODEL\x10\x01\x32\x94\x06\n\x04\x43\x61se\x12\x34\n\x0cGetGridCount\x12\x11.rips.CaseRequest\x1a\x0f.rips.GridCount\"\x00\x12\x38\n\x0cGetCellCount\x12\x15.rips.CellInfoRequest\x1a\x0f.rips.CellCount\"\x00\x12K\n\x19GetCellInfoForActiveCells\x12\x15.rips.CellInfoRequest\x1a\x13.rips.CellInfoArray\"\x00\x30\x01\x12K\n\x1bGetCellCenterForActiveCells\x12\x15.rips.CellInfoRequest\x1a\x11.rips.CellCenters\"\x00\x30\x01\x12Q\n\x1cGetCellCornersForActiveCells\x12\x15.rips.CellInfoRequest\x1a\x16.rips.CellCornersArray\"\x00\x30\x01\x12H\n\x16GetCoarseningInfoArray\x12\x11.rips.CaseRequest\x1a\x19.rips.CoarseningInfoArray\"\x00\x12\x38\n\x0cGetTimeSteps\x12\x11.rips.CaseRequest\x1a\x13.rips.TimeStepDates\"\x00\x12>\n\x10GetSelectedCells\x12\x11.rips.CaseRequest\x1a\x13.rips.SelectedCells\"\x00\x30\x01\x12>\n\x11GetDaysSinceStart\x12\x11.rips.CaseRequest\x1a\x14.rips.DaysSinceStart\"\x00\x12\x32\n\x0bGetCaseInfo\x12\x11.rips.CaseRequest\x1a\x0e.rips.CaseInfo\"\x00\x12\x34\n\x0cGetPdmObject\x12\x11.rips.CaseRequest\x1a\x0f.rips.PdmObject\"\x00\x12\x41\n\x17GetReservoirBoundingBox\x12\x11.rips.CaseRequest\x1a\x11.rips.BoundingBox\"\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Case_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_POROSITYMODELTYPE']._serialized_start=1244
  _globals['_POROSITYMODELTYPE']._serialized_end=1301
  _globals['_CASEREQUEST']._serialized_start=56
  _globals['_CASEREQUEST']._serialized_end=81
  _globals['_CASEINFO']._serialized_start=83
  _globals['_CASEINFO']._serialized_end=151
  _globals['_CASEINFOARRAY']._serialized_start=153
  _globals['_CASEINFOARRAY']._serialized_end=198
  _globals['_BOUNDINGBOX']._serialized_start=200
  _globals['_BOUNDINGBOX']._serialized_end=303
  _globals['_CASEGROUP']._serialized_start=305
  _globals['_CASEGROUP']._serialized_end=342
  _globals['_CASEGROUPS']._serialized_start=344
  _globals['_CASEGROUPS']._serialized_end=394
  _globals['_GRIDCOUNT']._serialized_start=396
  _globals['_GRIDCOUNT']._serialized_end=422
  _globals['_CELLCOUNT']._serialized_start=424
  _globals['_CELLCOUNT']._serialized_end=492
  _globals['_CELLINFOREQUEST']._serialized_start=494
  _globals['_CELLINFOREQUEST']._serialized_end=601
  _globals['_CELLINFOARRAY']._serialized_start=603
  _globals['_CELLINFOARRAY']._serialized_end=648
  _globals['_CELLINFO']._serialized_start=651
  _globals['_CELLINFO']._serialized_end=803
  _globals['_COARSENINGINFOARRAY']._serialized_start=805
  _globals['_COARSENINGINFOARRAY']._serialized_end=862
  _globals['_COARSENINGINFO']._serialized_start=864
  _globals['_COARSENINGINFO']._serialized_end=932
  _globals['_TIMESTEPDATES']._serialized_start=934
  _globals['_TIMESTEPDATES']._serialized_end=984
  _globals['_TIMESTEPDATE']._serialized_start=986
  _globals['_TIMESTEPDATE']._serialized_end=1088
  _globals['_DAYSSINCESTART']._serialized_start=1090
  _globals['_DAYSSINCESTART']._serialized_end=1128
  _globals['_SELECTEDCELL']._serialized_start=1130
  _globals['_SELECTEDCELL']._serialized_end=1190
  _globals['_SELECTEDCELLS']._serialized_start=1192
  _globals['_SELECTEDCELLS']._serialized_end=1242
  _globals['_CASE']._serialized_start=1304
  _globals['_CASE']._serialized_end=2092
# @@protoc_insertion_point(module_scope)



# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()
# Example code
print("ResInsight version: " + resinsight.version_string())

case = resinsight.project.case(case_id=0)
case.replace(
    new_grid_file="C:/Users/lindkvis/Projects/ResInsight/TestModels/Case_with_10_timesteps/Real0/BRUGGE_0000.EGRID"
)



# pylint: disable=no-self-use
"""
ResInsight caf::PdmObject connection module
"""

from functools import partial, wraps
import grpc
import re
import builtins
import importlib
import inspect
import sys

import PdmObject_pb2
import PdmObject_pb2_grpc
import Commands_pb2
import Commands_pb2_grpc


def camel_to_snake(name):
    s1 = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s1).lower()


def snake_to_camel(name):
    return "".join(word.title() for word in name.split("_"))


def add_method(cls):
    def decorator(func):
        setattr(cls, func.__name__, func)
        return func  # returning func means func can still be used normally

    return decorator


def add_static_method(cls):
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)

        setattr(cls, func.__name__, wrapper)
        # Note we are not binding func, but wrapper which accepts self but does exactly the same as func
        return func  # returning func means func can still be used normally

    return decorator


class PdmObjectBase:
    """
    The ResInsight base class for the Project Data Model
    """

    def _execute_command(self, **command_params):
        self.__warnings = []
        response, call = self._commands.Execute.with_call(
            Commands_pb2.CommandParams(**command_params)
        )
        for key, value in call.trailing_metadata():
            value = value.replace(";;", "\n")
            if key == "warning":
                self.__warnings.append(value)

        return response

    def __init__(self, pb2_object, channel):
        self.__warnings = []
        self.__chunk_size = 8160

        self._channel = channel

        # Create stubs
        if self._channel:
            self._pdm_object_stub = PdmObject_pb2_grpc.PdmObjectServiceStub(
                self._channel
            )
            self._commands = Commands_pb2_grpc.CommandsStub(self._channel)

        if pb2_object is not None:
            # Copy parameters from ResInsight
            assert isinstance(pb2_object, PdmObject_pb2.PdmObject)
            self._pb2_object = pb2_object
            for camel_keyword in self._pb2_object.parameters:
                snake_keyword = camel_to_snake(camel_keyword)
                setattr(self, snake_keyword, self.__get_grpc_value(camel_keyword))
        else:
            # Copy parameters from PdmObject defaults
            self._pb2_object = PdmObject_pb2.PdmObject(
                class_keyword=self.__class__.__name__
            )
            self.__copy_to_pb2()

    def copy_from(self, object):
        """Copy attribute values from object to self"""
        for attribute in dir(object):
            if not attribute.startswith("__"):
                value = getattr(object, attribute)
                # This is crucial to avoid overwriting methods
                if not callable(value):
                    setattr(self, attribute, value)
        if self.__custom_init__ is not None:
            self.__custom_init__(self._pb2_object, self._channel)
        self.update()

    def warnings(self):
        return self.__warnings

    def has_warnings(self):
        return len(self.__warnings) > 0

    def __copy_to_pb2(self):
        if self._pb2_object is not None:
            for snake_kw in dir(self):
                if not snake_kw.startswith("_"):
                    value = getattr(self, snake_kw)
                    # This is crucial to avoid overwriting methods
                    if not callable(value):
                        camel_kw = snake_to_camel(snake_kw)
                        self.__set_grpc_value(camel_kw, value)

    def pb2_object(self):
        """Private method"""
        return self._pb2_object

    def channel(self):
        """Private method"""
        return self._channel

    def address(self):
        """Get the unique address of the PdmObject

        Returns:
            A 64-bit unsigned integer address
        """

        return self._pb2_object.address

    def set_visible(self, visible):
        """Set the visibility of the object in the ResInsight project tree"""
        self._pb2_object.visible = visible

    def visible(self):
        """Get the visibility of the object in the ResInsight project tree"""
        return self._pb2_object.visible

    def print_object_info(self):
        """Print the structure and data content of the PdmObject"""
        print("=========== " + self.__class__.__name__ + " =================")
        print("Object Attributes: ")
        for snake_kw in dir(self):
            if not snake_kw.startswith("_") and not callable(getattr(self, snake_kw)):
                camel_kw = snake_to_camel(snake_kw)
                print(
                    "   "
                    + snake_kw
                    + " ["
                    + type(getattr(self, snake_kw)).__name__
                    + "]: "
                    + str(getattr(self, snake_kw))
                )
        print("Object Methods:")
        for snake_kw in dir(self):
            if not snake_kw.startswith("_") and callable(getattr(self, snake_kw)):
                print("   " + snake_kw)

    def __convert_from_grpc_value(self, value):
        if value.lower() == "false":
            return False
        if value.lower() == "true":
            return True
        try:
            int_val = int(value)
            return int_val
        except ValueError:
            try:
                float_val = float(value)
                return float_val
            except ValueError:
                # We may have a string. Strip internal start and end quotes
                value = value.strip('"')
                if self.__islist(value):
                    return self.__makelist(value)
                return value

    def __convert_to_grpc_value(self, value):
        if isinstance(value, bool):
            if value:
                return "true"
            return "false"
        if isinstance(value, PdmObjectBase):
            return value.__class__.__name__ + ":" + str(value.address())
        if isinstance(value, list):
            list_of_values = []
            for val in value:
                list_of_values.append(self.__convert_to_grpc_value(val))
            return "[" + ", ".join(list_of_values) + "]"
        return str(value)

    def __get_grpc_value(self, camel_keyword):
        return self.__convert_from_grpc_value(
            self._pb2_object.parameters[camel_keyword]
        )

    def __set_grpc_value(self, camel_keyword, value):
        self._pb2_object.parameters[camel_keyword] = self.__convert_to_grpc_value(value)

    def set_value(self, snake_keyword, value):
        """Set the value associated with the provided keyword and updates ResInsight

        Arguments:
            keyword(str): A string containing the parameter keyword
            value(varying): A value matching the type of the parameter.
                See keyword documentation and/or print_object_info() to find
                the correct data type.
        """
        setattr(self, snake_keyword, value)
        self.update()

    def __islist(self, value):
        return value.startswith("[") and value.endswith("]")

    def __makelist(self, list_string):
        list_string = list_string.lstrip("[")
        list_string = list_string.rstrip("]")
        if not list_string:
            # Return empty list if empty string. Otherwise, the split function will return ['']
            return []
        strings = list_string.split(", ")
        values = []
        for string in strings:
            values.append(self.__convert_from_grpc_value(string))
        return values

    def __from_pb2_to_resinsight_classes(self, pb2_object_list, super_class_definition):
        pdm_object_list = []
        from .generated.generated_classes import class_from_keyword

        for pb2_object in pb2_object_list:
            child_class_definition = class_from_keyword(pb2_object.class_keyword)
            if child_class_definition is None:
                child_class_definition = super_class_definition

            pdm_object = child_class_definition(
                pb2_object=pb2_object, channel=self.channel()
            )
            pdm_object_list.append(pdm_object)
        return pdm_object_list

    def descendants(self, class_definition):
        """Get a list of all project tree descendants matching the class keyword
        Arguments:
            class_definition[class]: A class definition matching the type of class wanted

        Returns:
            A list of PdmObjects matching the class_definition
        """
        assert inspect.isclass(class_definition)

        class_keyword = class_definition.__name__
        try:
            request = PdmObject_pb2.PdmDescendantObjectRequest(
                object=self._pb2_object, child_keyword=class_keyword
            )
            object_list = self._pdm_object_stub.GetDescendantPdmObjects(request).objects
            return self.__from_pb2_to_resinsight_classes(object_list, class_definition)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return []  # Valid empty result
            raise e

    def children(self, child_field, class_definition):
        """Get a list of all direct project tree children inside the provided child_field
        Arguments:
            child_field[str]: A field name
        Returns:
            A list of PdmObjects inside the child_field
        """
        request = PdmObject_pb2.PdmChildObjectRequest(
            object=self._pb2_object, child_field=child_field
        )
        try:
            object_list = self._pdm_object_stub.GetChildPdmObjects(request).objects
            return self.__from_pb2_to_resinsight_classes(object_list, class_definition)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return []
            raise e

    def add_new_object(self, class_definition, child_field=""):
        """Create and add an object to the specified child field
        Arguments:
            class_definition[class]: Class definition of the object to create
            child_field[str]: The keyword for the field to create a new object in. If empty, the first child array field is used.
        Returns:
            The created PdmObject inside the child_field
        """
        from .generated.generated_classes import class_from_keyword

        assert inspect.isclass(class_definition)

        class_keyword = class_definition.__name__

        request = PdmObject_pb2.CreatePdmChildObjectRequest(
            object=self._pb2_object,
            child_field=child_field,
            class_keyword=class_keyword,
        )
        try:
            pb2_object = self._pdm_object_stub.CreateChildPdmObject(request)
            child_class_definition = class_from_keyword(pb2_object.class_keyword)

            if child_class_definition is None:
                child_class_definition = class_keyword

            pdm_object = child_class_definition(
                pb2_object=pb2_object, channel=self.channel()
            )
            return pdm_object
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return None
            raise e

    def ancestor(self, class_definition):
        """Find the first ancestor that matches the provided class_keyword
        Arguments:
            class_definition[class]: A class definition matching the type of class wanted
        """
        assert inspect.isclass(class_definition)

        class_keyword = class_definition.__name__
        from .generated.generated_classes import class_from_keyword

        request = PdmObject_pb2.PdmParentObjectRequest(
            object=self._pb2_object, parent_keyword=class_keyword
        )
        try:
            pb2_object = self._pdm_object_stub.GetAncestorPdmObject(request)
            child_class_definition = class_from_keyword(pb2_object.class_keyword)

            if child_class_definition is None:
                child_class_definition = class_definition

            pdm_object = child_class_definition(
                pb2_object=pb2_object, channel=self.channel()
            )
            return pdm_object
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                return None
            raise e

    def _call_get_method_async(self, method_name):
        request = PdmObject_pb2.PdmObjectGetterRequest(
            object=self._pb2_object, method=method_name
        )
        for chunk in self._pdm_object_stub.CallPdmObjectGetter(request):
            yield chunk

    def _call_get_method(self, method_name):
        all_values = []
        generator = self._call_get_method_async(method_name)
        for chunk in generator:
            data = getattr(chunk, chunk.WhichOneof("data"))
            for value in data.data:
                all_values.append(value)
        return all_values

    def __generate_set_method_chunks(self, array, method_request):
        index = -1

        while index < len(array):
            chunk = PdmObject_pb2.PdmObjectSetterChunk()
            if index == -1:
                chunk.set_request.CopyFrom(
                    PdmObject_pb2.PdmObjectSetterRequest(
                        request=method_request, data_count=len(array)
                    )
                )
                index += 1
            else:
                actual_chunk_size = min(len(array) - index + 1, self.__chunk_size)
                if isinstance(array[0], float):
                    chunk.CopyFrom(
                        PdmObject_pb2.PdmObjectSetterChunk(
                            doubles=PdmObject_pb2.DoubleArray(
                                data=array[index : index + actual_chunk_size]
                            )
                        )
                    )
                elif isinstance(array[0], int):
                    chunk.CopyFrom(
                        PdmObject_pb2.PdmObjectSetterChunk(
                            ints=PdmObject_pb2.IntArray(
                                data=array[index : index + actual_chunk_size]
                            )
                        )
                    )
                elif isinstance(array[0], str):
                    chunk.CopyFrom(
                        PdmObject_pb2.PdmObjectSetterChunk(
                            strings=PdmObject_pb2.StringArray(
                                data=array[index : index + actual_chunk_size]
                            )
                        )
                    )
                else:
                    raise Exception("Wrong data type for set method")
                index += actual_chunk_size
            yield chunk
        # Final empty message to signal completion
        chunk = PdmObject_pb2.PdmObjectSetterChunk()
        yield chunk

    def _call_set_method(self, method_name, values):
        method_request = PdmObject_pb2.PdmObjectGetterRequest(
            object=self._pb2_object, method=method_name
        )
        request_iterator = self.__generate_set_method_chunks(values, method_request)
        reply = self._pdm_object_stub.CallPdmObjectSetter(request_iterator)
        if reply.accepted_value_count < len(values):
            raise IndexError

    def _call_pdm_method(self, method_name, **kwargs):
        pb2_params = PdmObject_pb2.PdmObject(class_keyword=method_name)
        for key, value in kwargs.items():
            pb2_params.parameters[snake_to_camel(key)] = self.__convert_to_grpc_value(
                value
            )
        request = PdmObject_pb2.PdmObjectMethodRequest(
            object=self._pb2_object, method=method_name, params=pb2_params
        )

        pb2_object = self._pdm_object_stub.CallPdmObjectMethod(request)

        from .generated.generated_classes import class_from_keyword

        child_class_definition = class_from_keyword(pb2_object.class_keyword)
        if child_class_definition is None:
            return None

        pdm_object = child_class_definition(
            pb2_object=pb2_object, channel=self.channel()
        )
        return pdm_object

    def update(self):
        """Sync all fields from the Python Object to ResInsight"""
        self.__copy_to_pb2()
        if self._pdm_object_stub is not None:
            self._pdm_object_stub.UpdateExistingPdmObject(self._pb2_object)
        else:
            raise Exception(
                "Object is not connected to GRPC service so cannot update ResInsight"
            )



# Access to environment variables and path tools
import os

# Load ResInsight Processing Server Client Library
import rips

# Connect to ResInsight instance
resinsight = rips.Instance.find()

# This requires the TestModels to be installed with ResInsight (RESINSIGHT_BUNDLE_TESTMODELS):
resinsight_exe_path = os.environ.get("RESINSIGHT_EXECUTABLE")

# Get the TestModels path from the executable path
resinsight_install_path = os.path.dirname(resinsight_exe_path)
test_models_path = os.path.join(resinsight_install_path, "TestModels")
path_name = os.path.join(
    test_models_path, "TEST10K_FLT_LGR_NNC/TEST10K_FLT_LGR_NNC.EGRID"
)
case = resinsight.project.load_case(path_name)
case.create_view()

# Print out lots of information from the case object
print("Case id: " + str(case.id))
print("Case name: " + case.name)
print("Case type: " + case.__class__.__name__)
print("Case file name: " + case.file_path)
print("Case reservoir bounding box:", case.reservoir_boundingbox())

timesteps = case.time_steps()
for t in timesteps:
    print("Year: " + str(t.year))
    print("Month: " + str(t.month))
